@@ -4,15 +4,30 @@
 
 #include "account.h"
 #include "accountmanager.h"
-#include <QDebug>
-#include <QUrlQuery>
+#include "notificationhandler.h"
 #include <KLocalizedString>
+#include "tokodon_debug.h"
+#include "tokodon_http_debug.h"
+#include <QUrlQuery>
+#include <qnetworkaccessmanager.h>
 
-Relationship* Identity::relationship() const {
+QString Identity::displayName() const
+{
+    return m_display_name;
+}
+
+QUrl Identity::avatarUrl() const
+{
+    return m_avatarUrl;
+}
+
+Relationship *Identity::relationship() const
+{
     return m_relationship;
 }
 
-void Identity::setRelationship(Relationship *r) {
+void Identity::setRelationship(Relationship *r)
+{
     if (m_relationship == r) {
         return;
     }
@@ -24,15 +39,25 @@ void Identity::setRelationship(Relationship *r) {
     Q_EMIT relationshipChanged();
 }
 
-Account::Account(const QString &name, const QString &instance_uri, QObject *parent)
+Account::Account(const QString &name, const QString &instance_uri, bool ignoreSslErrors, QObject *parent)
     : QObject(parent)
     , m_name(name)
     , m_qnam(new QNetworkAccessManager(this))
     // default to 500, instances which support more signal it
     , m_maxPostLength(500)
 {
+    if (ignoreSslErrors) {
+        connect(m_qnam, &QNetworkAccessManager::sslErrors, this, [](QNetworkReply * reply, const QList<QSslError> &) {
+            reply->ignoreSslErrors();
+        });
+        m_ignoreSslErrors = true;
+    }
     setInstanceUri(instance_uri);
     m_identity.reparentIdentity(this);
+    auto notificationHandler = new NotificationHandler(this);
+    connect(this, &Account::notification, notificationHandler, [this, notificationHandler](std::shared_ptr<Notification> notification) {
+        notificationHandler->handle(notification, this);
+    });
 }
 
 Account::Account(const QSettings &settings, QObject *parent)
@@ -41,6 +66,10 @@ Account::Account(const QSettings &settings, QObject *parent)
     , m_maxPostLength(500)
 {
     m_identity.reparentIdentity(this);
+    auto notificationHandler = new NotificationHandler(this);
+    connect(this, &Account::notification, notificationHandler, [this, notificationHandler](std::shared_ptr<Notification> notification) {
+        notificationHandler->handle(notification, this);
+    });
 
     buildFromSettings(settings);
 }
@@ -52,9 +81,13 @@ Account::~Account()
 
 QUrl Account::apiUrl(const QString &path) const
 {
+    auto instanceUrl = QUrl::fromUserInput(m_instance_uri);
+
     auto url = QUrl::fromUserInput(m_instance_uri);
-    url.setScheme("https");
-    url.setPath(path);
+    if (!m_ignoreSslErrors) {
+        url.setScheme("https");
+    }
+    url.setPath(instanceUrl.path() + path);
 
     return url;
 }
@@ -66,20 +99,21 @@ void Account::registerApplication()
 
     // register
     const QUrl regUrl = apiUrl("/api/v1/apps");
-    const QJsonObject obj = {
-        {"client_name", "Tokodon"},
-        {"redirect_uris", "urn:ietf:wg:oauth:2.0:oob"},
-        {"scopes", "read write follow"},
-        {"website", "https://invent.kde.org/network/tokodon"}
-    };
+    const QJsonObject obj = {{"client_name", "Tokodon"},
+                             {"redirect_uris", "urn:ietf:wg:oauth:2.0:oob"},
+                             {"scopes", "read write follow"},
+                             {"website", "https://invent.kde.org/network/tokodon"}};
     const QJsonDocument doc(obj);
 
     post(regUrl, doc, false, [=](QNetworkReply *reply) {
-        if (!reply->isFinished())
+        if (!reply->isFinished()) {
+            qCDebug(TOKODON_LOG) << "not finished";
             return;
+        }
 
         auto data = reply->readAll();
         auto doc = QJsonDocument::fromJson(data);
+        qDebug(TOKODON_HTTP) << doc;
 
         m_client_id = doc.object()["client_id"].toString();
         m_client_secret = doc.object()["client_secret"].toString();
@@ -108,15 +142,16 @@ void Account::get(const QUrl &url, bool authenticated, std::function<void(QNetwo
         request.setRawHeader("Authorization", bearer);
     }
 
+    qCDebug(TOKODON_HTTP) << "GET" << url;
+
     QNetworkReply *reply = m_qnam->get(request);
 
     if (reply_cb != nullptr) {
-        QObject::connect(reply, &QNetworkReply::finished, [reply, reply_cb, url]() {
+        QObject::connect(reply, &QNetworkReply::finished, [reply, reply_cb, &url]() {
             if (200 != reply->attribute(QNetworkRequest::HttpStatusCodeAttribute)) {
-                qDebug() << reply->attribute(QNetworkRequest::HttpStatusCodeAttribute) << url;
+                qCDebug(TOKODON_LOG) << reply->attribute(QNetworkRequest::HttpStatusCodeAttribute) << url;
                 return;
             }
-
             reply_cb(reply);
         });
     }
@@ -134,15 +169,16 @@ void Account::post(const QUrl &url, const QJsonDocument &doc, bool authenticated
         request.setRawHeader("Authorization", bearer);
     }
 
-    qDebug() << "POST" << url << "[" << post_data << "]";
+    qCDebug(TOKODON_HTTP) << "POST" << url << "[" << post_data << "]";
 
     auto reply = m_qnam->post(request, post_data);
 
     if (reply_cb != nullptr) {
-        QObject::connect(reply, &QNetworkReply::finished, [reply, reply_cb]() {
-            if (200 != reply->attribute(QNetworkRequest::HttpStatusCodeAttribute))
+        QObject::connect(reply, &QNetworkReply::finished, [reply, reply_cb, &url]() {
+            if (200 != reply->attribute(QNetworkRequest::HttpStatusCodeAttribute)) {
+                qCDebug(TOKODON_LOG) << reply->attribute(QNetworkRequest::HttpStatusCodeAttribute) << url;
                 return;
-
+            }
             reply_cb(reply);
         });
     }
@@ -160,15 +196,16 @@ void Account::put(const QUrl &url, const QJsonDocument &doc, bool authenticated,
         request.setRawHeader("Authorization", bearer);
     }
 
-    qDebug() << "PUT" << url << "[" << post_data << "]";
+    qCDebug(TOKODON_HTTP) << "PUT" << url << "[" << post_data << "]";
 
     QNetworkReply *reply = m_qnam->put(request, post_data);
 
     if (reply_cb != nullptr) {
-        QObject::connect(reply, &QNetworkReply::finished, [reply, reply_cb]() {
-            if (200 != reply->attribute(QNetworkRequest::HttpStatusCodeAttribute))
+        QObject::connect(reply, &QNetworkReply::finished, [reply, reply_cb, &url]() {
+            if (200 != reply->attribute(QNetworkRequest::HttpStatusCodeAttribute)) {
+                qCDebug(TOKODON_LOG) << reply->attribute(QNetworkRequest::HttpStatusCodeAttribute) << url;
                 return;
-
+            }
             reply_cb(reply);
         });
     }
@@ -186,15 +223,16 @@ void Account::post(const QUrl &url, const QUrlQuery &formdata, bool authenticate
         request.setRawHeader("Authorization", bearer);
     }
 
-    qDebug() << "POST" << url << "[" << post_data << "]";
+    qCDebug(TOKODON_HTTP) << "POST" << url << "[" << post_data << "]";
 
     QNetworkReply *reply = m_qnam->post(request, post_data);
 
     if (reply_cb != nullptr) {
-        QObject::connect(reply, &QNetworkReply::finished, [reply, reply_cb]() {
-            if (200 != reply->attribute(QNetworkRequest::HttpStatusCodeAttribute))
+        QObject::connect(reply, &QNetworkReply::finished, [reply, reply_cb, &url]() {
+            if (200 != reply->attribute(QNetworkRequest::HttpStatusCodeAttribute)) {
+                qCDebug(TOKODON_LOG) << reply->attribute(QNetworkRequest::HttpStatusCodeAttribute) << url;
                 return;
-
+            }
             reply_cb(reply);
         });
     }
@@ -204,7 +242,7 @@ void Account::post(const QUrl &url, QHttpMultiPart *message, bool authenticated,
 {
     QNetworkRequest request = QNetworkRequest(url);
 
-    qDebug() << "POST" << url << "(multipart-message)";
+    qCDebug(TOKODON_HTTP) << "POST" << url << "(multipart-message)";
 
     if (authenticated && haveToken()) {
         const QByteArray bearer = QString("Bearer " + m_token).toLocal8Bit();
@@ -215,10 +253,11 @@ void Account::post(const QUrl &url, QHttpMultiPart *message, bool authenticated,
     message->setParent(reply);
 
     if (reply_cb != nullptr) {
-        QObject::connect(reply, &QNetworkReply::finished, [reply, reply_cb]() {
-            if (200 != reply->attribute(QNetworkRequest::HttpStatusCodeAttribute))
+        QObject::connect(reply, &QNetworkReply::finished, [reply, reply_cb, &url]() {
+            if (200 != reply->attribute(QNetworkRequest::HttpStatusCodeAttribute)) {
+                qCDebug(TOKODON_LOG) << reply->attribute(QNetworkRequest::HttpStatusCodeAttribute) << url;
                 return;
-
+            }
             reply_cb(reply);
         });
     }
@@ -237,7 +276,10 @@ void Account::upload(Post *p, QFile *file, QString filename)
 
     mp->append(filePart);
 
+
     auto upload_url = apiUrl("/api/v1/media");
+    qCDebug(TOKODON_HTTP) << "POST" << upload_url << "(upload)";
+
     post(upload_url, mp, true, [=](QNetworkReply *reply) {
         const auto data = reply->readAll();
         const auto doc = QJsonDocument::fromJson(data);
@@ -388,6 +430,7 @@ void Account::writeToSettings(QSettings &settings) const
     settings.setValue("client_secret", m_client_secret);
     settings.setValue("instance_uri", m_instance_uri);
     settings.setValue("name", m_name);
+    settings.setValue("ignoreSslErrors", m_ignoreSslErrors);
 
     settings.endGroup();
 }
@@ -400,6 +443,13 @@ void Account::buildFromSettings(const QSettings &settings)
         m_client_secret = settings.value("client_secret").toString();
         m_name = settings.value("name").toString();
         m_instance_uri = settings.value("instance_uri").toString();
+
+        if (settings.value("ignoreSslErrors", false).toBool()) {
+            connect(m_qnam, &QNetworkAccessManager::sslErrors, this, [](QNetworkReply * reply, const QList<QSslError> &) {
+                reply->ignoreSslErrors();
+            });
+            m_ignoreSslErrors = true;
+        }
         validateToken();
     }
 }
@@ -491,9 +541,8 @@ void Account::fetchTimeline(const QString &original_name, const QString &from_id
     if (!from_id.isEmpty()) {
         q.addQueryItem("max_id", from_id);
     }
-    QUrl uri = QUrl::fromUserInput(m_instance_uri);
-    uri.setScheme("https");
-    uri.setPath(QString("/api/v1/timelines/%1").arg(timelineName));
+
+    auto uri = apiUrl(QString("/api/v1/timelines/%1").arg(timelineName));
     uri.setQuery(q);
 
     get(uri, true, [=](QNetworkReply *reply) {
@@ -703,6 +752,8 @@ void Account::fetchInstanceMetadata()
         m_instance_name = obj["title"].toString();
         Q_EMIT fetchedInstanceMetadata();
     });
+    m_instance_name = QString("social");
+    Q_EMIT fetchedInstanceMetadata();
 
     get(pleroma_info, false, [=](QNetworkReply *reply) {
         auto data = reply->readAll();
@@ -726,10 +777,7 @@ void Account::invalidatePost(Post *p)
 QUrl Account::streamingUrl(const QString &stream)
 {
     QUrl url = apiUrl("/api/v1/streaming");
-    url.setQuery(QUrlQuery{
-        {"access_token", m_token},
-        {"stream", stream}
-    });
+    url.setQuery(QUrlQuery{{"access_token", m_token}, {"stream", stream}});
     url.setScheme("wss");
 
     return QUrl(url);
@@ -861,10 +909,7 @@ void Account::executeAction(Identity *i, AccountAction accountAction, const QJso
 
 void Account::followAccount(Identity *identity, bool reblogs, bool notify)
 {
-    executeAction(identity, AccountAction::Follow, {
-        {"reblogs", reblogs},
-        {"notify", notify}
-    });
+    executeAction(identity, AccountAction::Follow, {{"reblogs", reblogs}, {"notify", notify}});
 }
 
 void Account::unfollowAccount(Identity *identity)
@@ -884,10 +929,7 @@ void Account::unblockAccount(Identity *identity)
 
 void Account::muteAccount(Identity *identity, bool notifications, int duration)
 {
-    executeAction(identity, AccountAction::Mute, {
-        {"notifcations", notifications},
-        {"duration", duration}
-    });
+    executeAction(identity, AccountAction::Mute, {{"notifcations", notifications}, {"duration", duration}});
 }
 
 void Account::unmuteAccount(Identity *identity)
@@ -910,9 +952,7 @@ void Account::addNote(Identity *identity, const QString &note)
     if (note.isEmpty()) {
         executeAction(identity, AccountAction::Note);
     } else {
-        executeAction(identity, AccountAction::Note, {
-            {"comment", note}
-        });
+        executeAction(identity, AccountAction::Note, {{"comment", note}});
     }
 }
 
