@@ -21,6 +21,7 @@
 #include <KAboutData>
 #ifdef HAVE_KDBUSADDONS
 #include <KDBusService>
+#include <KWindowSystem>
 #endif
 #include <KConfig>
 #include <KLocalizedContext>
@@ -34,6 +35,7 @@
 #include "attachmenteditormodel.h"
 #include "clipboard.h"
 #include "config.h"
+#include "notificationmodel.h"
 #include "post.h"
 #include "timelinemodel.h"
 
@@ -91,6 +93,7 @@ int main(int argc, char *argv[])
     qmlRegisterSingletonInstance("org.kde.kmasto", 1, 0, "Config", config);
     qmlRegisterSingletonInstance("org.kde.kmasto", 1, 0, "AccountManager", &AccountManager::instance());
     qmlRegisterType<TimelineModel>("org.kde.kmasto", 1, 0, "TimelineModel");
+    qmlRegisterType<NotificationModel>("org.kde.kmasto", 1, 0, "NotificationModel");
     qmlRegisterSingletonType<Clipboard>("org.kde.kmasto", 1, 0, "Clipboard", [](QQmlEngine *engine, QJSEngine *scriptEngine) -> QObject * {
         Q_UNUSED(engine)
         Q_UNUSED(scriptEngine)
@@ -101,6 +104,7 @@ int main(int argc, char *argv[])
     qRegisterMetaType<Identity *>("Identity*");
     qRegisterMetaType<Relationship *>("Relationship*");
     qmlRegisterUncreatableType<Post>("org.kde.kmasto", 1, 0, "Post", "ENUM");
+    qmlRegisterUncreatableType<Notification>("org.kde.kmasto", 1, 0, "Notification", "ENUM");
 
     QQmlApplicationEngine engine;
     engine.rootContext()->setContextObject(new KLocalizedContext(&engine));
@@ -127,8 +131,8 @@ int main(int argc, char *argv[])
         for (auto obj : rootObjects) {
             auto view = qobject_cast<QQuickWindow *>(obj);
             if (view) {
-                view->show();
-                view->raise();
+                KWindowSystem::updateStartupId(view);
+                KWindowSystem::activateWindow(view);
                 return;
             }
         }
