@@ -13,11 +13,53 @@
 #include <QUrl>
 
 #include <memory>
+#include <optional>
+#include <qjsonobject.h>
+#include <qobjectdefs.h>
 
 class Post;
 class Identity;
 class Account;
 
+class Card
+{
+    Q_GADGET
+    Q_PROPERTY(QString authorName READ authorName)
+    Q_PROPERTY(QString authorUrl READ authorUrl)
+    Q_PROPERTY(QString blurhash READ blurhash)
+    Q_PROPERTY(QString description READ description)
+    Q_PROPERTY(QString embedUrl READ embedUrl)
+    Q_PROPERTY(int width READ width)
+    Q_PROPERTY(int height READ height)
+    Q_PROPERTY(QString html READ html)
+    Q_PROPERTY(QString image READ image)
+    Q_PROPERTY(QString providerName READ providerName)
+    Q_PROPERTY(QString providerUrl READ providerUrl)
+    Q_PROPERTY(QString title READ title)
+    Q_PROPERTY(QUrl url READ url)
+
+public:
+    Card() = default;
+    Card(QJsonObject card);
+
+    QString authorName() const;
+    QString authorUrl() const;
+    QString blurhash() const;
+    QString description() const;
+    QString embedUrl() const;
+    int width() const;
+    int height() const;
+    QString html() const;
+    QString image() const;
+    QString providerName() const;
+    QString providerUrl() const;
+    QString title() const;
+    QUrl url() const;
+
+private:
+    QJsonObject m_card;
+};
+
 /// Post's attachment object.
 /// TODO make it possible to fetch the images with a Qml image provider.
 /// TODO use getter and setter
@@ -47,9 +89,17 @@ public:
     void setDescription(QString desc);
 };
 
-struct Notification {
-    Notification() = delete;
-    Notification(const Notification &) = delete;
+class Notification
+{
+    Q_GADGET
+
+public:
+    Notification()
+    {
+    }
+    Notification(const Notification &)
+    {
+    }
     Notification(Account *parent, QJsonObject &obj);
 
     enum Type {
@@ -57,17 +107,24 @@ struct Notification {
         Follow,
         Repeat,
         Favorite,
+        Poll,
+        FollowRequest,
     };
+    Q_ENUM(Type);
 
-    Account *m_account;
-    Type m_type;
-
-    std::shared_ptr<Post> m_post;
-    std::shared_ptr<Identity> m_identity;
     int id() const;
+    Account *account() const;
+    Type type() const;
+    std::shared_ptr<Post> post() const;
+    std::shared_ptr<Identity> identity() const;
 
 private:
     int m_id;
+
+    Account *m_account;
+    Type m_type;
+    std::shared_ptr<Post> m_post;
+    std::shared_ptr<Identity> m_identity;
 };
 
 class Post : public QObject
@@ -114,6 +171,8 @@ public:
     void setInReplyTo(const QString &inReplyTo);
     QStringList mentions() const;
     void setMentions(const QStringList &mentions);
+    std::optional<Card> card() const;
+    void setCard(std::optional<Card> card);
 
     void addAttachment(const QJsonObject &attachment);
     Q_INVOKABLE void uploadAttachment(const QUrl &filename);
@@ -163,4 +222,8 @@ Q_SIGNALS:
 
 private:
     QString m_replyTargetId;
+    std::optional<Card> m_card;
 };
+
+Q_DECLARE_METATYPE(Card)
+Q_DECLARE_METATYPE(Notification)
