@@ -3,13 +3,14 @@
 // SPDX-License-Identifier: GPL-3.0-only
 
 #include "timelinemodel.h"
+#include "abstracttimelinemodel.h"
 #include "accountmodel.h"
 #include "threadmodel.h"
 #include <KLocalizedString>
 #include <QtMath>
 
 TimelineModel::TimelineModel(QObject *parent)
-    : QAbstractListModel(parent)
+    : AbstractTimelineModel(parent)
     , m_last_fetch(time(nullptr))
 {
     connect(m_manager, &AccountManager::accountSelected, this, &TimelineModel::nameChanged);
@@ -215,6 +216,8 @@ QHash<int, QByteArray> TimelineModel::roleNames() const
         {UrlRole, QByteArrayLiteral("url")},
         {ThreadModelRole, QByteArrayLiteral("threadModel")},
         {AccountModelRole, QByteArrayLiteral("accountModel")},
+        {CardRole, QByteArrayLiteral("card")},
+        {TypeRole, QByteArrayLiteral("type")},
     };
 }
 
@@ -227,6 +230,8 @@ QVariant TimelineModel::data(const QModelIndex &index, int role) const
     auto p = m_timeline[row];
 
     switch (role) {
+    case TypeRole:
+        return false;
     case IdRole:
         return p->m_post_id;
     case Qt::DisplayRole:
@@ -271,6 +276,11 @@ QVariant TimelineModel::data(const QModelIndex &index, int role) const
         return p->m_subject;
     case AttachmentsRole:
         return QVariant::fromValue<QList<Attachment *>>(p->m_attachments);
+    case CardRole:
+        if (p->card().has_value()) {
+            return QVariant::fromValue<Card>(*p->card());
+        }
+        return false;
     case ThreadModelRole:
         return QVariant::fromValue<QAbstractListModel *>(new ThreadModel(m_manager, p->m_post_id));
     case AccountModelRole:
@@ -281,10 +291,10 @@ QVariant TimelineModel::data(const QModelIndex &index, int role) const
         if (secsTo < 60 * 60) {
             const auto hours = p->m_published_at.time().hour();
             const auto minutes = p->m_published_at.time().minute();
-            return i18nc(
-                    "hour:minute", "%1:%2",
-                    hours < 10 ? QChar('0') + QString::number(hours) : QString::number(hours),
-                    minutes < 10 ? QChar('0') + QString::number(minutes) : QString::number(minutes));
+            return i18nc("hour:minute",
+                         "%1:%2",
+                         hours < 10 ? QChar('0') + QString::number(hours) : QString::number(hours),
+                         minutes < 10 ? QChar('0') + QString::number(minutes) : QString::number(minutes));
         } else if (secsTo < 60 * 60 * 24) {
             return i18n("%1h", qCeil(secsTo / (60 * 60)));
         } else if (secsTo < 60 * 60 * 24 * 7) {
